/*******************************************************************************
 * Copyright 2012 pw999
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.oqt.test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

/**
 * This application will automatically create the integration tests for the maven-shitty-plugin
 * 
 */
public class App {
	private static List<String[]> versions = null;

	private static final File basePom = new File(App.class.getResource("/basePom.xml").getFile());
	private static final File baseGoals = new File(App.class.getResource("/baseGoals.txt").getFile());
	private static File outputDir;

	public static final String URL_H2 = "jdbc:h2:db/test/h2/junit_H2";
	public static final String URL_HSQL = "jdbc:hsqldb:file:db/test/hsql/junit_HSQL";
	public static final String URL_DERBY = "jdbc:derby:db/test/derby;create=true;databaseName=shitty";

	public static final String DRIVER_H2 = "org.h2.Driver";
	public static final String DRIVER_HSQL = "org.hsqldb.jdbcDriver";
	public static final String DRIVER_DERBY = "org.apache.derby.jdbc.EmbeddedDriver";

	public static final String DICTIONARY_H2 = "h2";
	public static final String DICTIONARY_HSQL = "hsql";
	public static final String DICTIONARY_DERBY = "derby";

	public static final String USER_H2 = "";
	public static final String USER_DERBY = "";
	public static final String USER_HSQL = "sa";

	private static void initList() {
	    versions = new ArrayList<String[]>();
		versions.add(new String[] { "1.0.3", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "1.0.4", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "1.2.0", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "1.2.1", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "2.0.1", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "2.1.1", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		versions.add(new String[] { "2.2.1", DRIVER_DERBY, URL_DERBY, DICTIONARY_DERBY, USER_DERBY });
		
		versions.add(new String[] { "1.0.3", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL});
		versions.add(new String[] { "1.0.4", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL});
		versions.add(new String[] { "1.2.0", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL});
		versions.add(new String[] { "1.2.1", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL });
		versions.add(new String[] { "2.0.1", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL });
		versions.add(new String[] { "2.1.1", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL });
		versions.add(new String[] { "2.2.1", DRIVER_HSQL, URL_HSQL, DICTIONARY_HSQL, USER_HSQL });

		//H2 is not supported by OpenJPA < 1.2.0
		versions.add(new String[] { "1.2.0", DRIVER_H2, URL_H2, DICTIONARY_H2, USER_H2});
		versions.add(new String[] { "1.2.1", DRIVER_H2, URL_H2, DICTIONARY_H2, USER_H2 });
		versions.add(new String[] { "2.0.1", DRIVER_H2, URL_H2, DICTIONARY_H2, USER_H2 });
		versions.add(new String[] { "2.1.1", DRIVER_H2, URL_H2, DICTIONARY_H2, USER_H2 });
		versions.add(new String[] { "2.2.1", DRIVER_H2, URL_H2, DICTIONARY_H2, USER_H2 });
	}

	public static void main(String[] args) throws IOException {
		initList();
		if (!basePom.exists()) {
			throw new IOException("File does not exist: " + basePom.getAbsolutePath());
		}
		if (!baseGoals.exists()) {
			throw new IOException("File does not exist: " + basePom.getAbsolutePath());
		}
		outputDir = new File(App.class.getResource("/").getFile() + "/../../src/it");
		System.out.println("Output dir will be: " + outputDir.getAbsolutePath());
		if (!outputDir.exists()) {
			outputDir.mkdir();
		}
		for (String[] version : versions) {
			File f = new File(outputDir.getAbsolutePath() + "/openjpa-" + version[0].replace(".", "_") + "_" + version[3]);

			if (!f.exists()) {
				f.mkdir();
			}
			File pom = new File(f.getAbsolutePath() + "/pom.xml");
			File goals = new File(f.getAbsolutePath() + "/goals.txt");

			BufferedReader br = null;
			BufferedWriter bw = null;
			try {
				// pom.xml
				Reader r = new FileReader(basePom);
				br = new BufferedReader(r);

				Writer w = new FileWriter(pom);
				bw = new BufferedWriter(w);

				String s = null;
				while ((s = br.readLine()) != null) {
					if (Pattern.compile("\\{[0-9]{1}\\}").matcher(s).find()) {
						bw.write(s.replace("{0}", version[0]).replace("{1}", version[1]).replace("{2}", version[2]).replace("{3}", version[3]).replace("{4}", version[4]));
					} else {
						bw.write(s);
					}
					bw.write("\r\n");
				}
				bw.flush();

				// goals.txt
				r = new FileReader(baseGoals);
				br = new BufferedReader(r);

				w = new FileWriter(goals);
				bw = new BufferedWriter(w);

				s = null;
				while ((s = br.readLine()) != null) {
					bw.write(s);
					bw.write("\r\n");
				}
				bw.flush();
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				bw.close();
				br.close();
			}
		}
		System.out.println("Finished !");
	}
}
